var esriMapsSettings = (function () {    
    _loadScript = function (InitScriptPath) {
        var script = document.createElement("script");
        script.type = "text/javascript";
        script.src = InitScriptPath;
        document.getElementsByTagName("head")[0].appendChild(script);
    };
    _getConfigSettingsAndInitialize = function () {
        var appUrl = esriMapsSettings.getQueryStringParameter("SPAppWebUrl");
        var ctx = esriMapsSettings.getClientContext();
        var clientContext = ctx.clientContext;        
        var configurationSettingsListTitle, valueColumnTitle;
        if (ctx.isSPO) {
            valueColumnTitle = 'ConfigValue';
        } else {            
            valueColumnTitle = 'Value';            
        }
        
        var items = ctx.configList.getItems(SP.CamlQuery.createAllItemsQuery());
        clientContext.load(items);
        clientContext.executeQueryAsync(
            Function.createDelegate(this, function (sender, args) {
                var enumerator = items.getEnumerator();
                while (enumerator.moveNext()) {
                    var li = enumerator.get_current();
                    var title = li.get_item("Title");
                    if (title === "EsriMapsWebRoot") {
                        var url = li.get_item(valueColumnTitle);
                        esriMapsSettings.esriMapsWebRoot = url;
                        EsriMapsWebRoot = url;
                    }
                    if (title === "Proxy" || title === "EsriMapsProxy") {
                        var proxy = li.get_item(valueColumnTitle);
                        esriMapsSettings.esriMapsWebProxyUrl = proxy;
                    }
                    if (title === "ArcGISConnection") {
                        var portalUrl = li.get_item(valueColumnTitle);
                        esriMapsSettings.arcGISConnection = esriMapsSettings.standardizePortalUrl(portalUrl);
                    }
                } 
                if (!esriMapsSettings.esriMapsWebRoot) { // this is only for ManageExtensions.aspx
                     var port = document.location.port;
                     var siteUrl = esriMapsSettings.getQueryStringParameter("SiteUrlRelative") || "";
                     var origin = document.location.protocol + "//" + document.location.host; 
                     var hivePrefix = _spPageContextInfo.layoutsUrl?"/15":"";
                     var url =  origin + siteUrl + "/_layouts" + hivePrefix + "/ESRI.ArcGIS.Mapping.SharePoint";
                     esriMapsSettings.esriMapsWebRoot = url;
                     EsriMapsWebRoot = url;
                }

                _loadScript(_getInitFilePath());
            }),
            Function.createDelegate(this, function (sender, args) {
                console.log(args);
            }));
    };
    _getInitFilePath = function () {
        return esriMapsSettings.getsharePointJSPath() + "/init.js";
    };

    return {
        esriMapsWebRoot: "",
        esriMapsWebProxyUrl: "",

        standardizePortalUrl: function (portalUrl) {
            var portal = portalUrl.replace(/\/$/, "");
            if (/^http(s)?:\/\//.test(portal)) {
                return portal;   // if portalUrl starts with http(s) and then hornor it.
            } else {
                return document.location.protocol + "//" + portal.replace(/^(\/\/)/, "");  // if not, use the document protocol;
            }            
        },

        initFromScriptPath: function (esriMapsWebRoot) {
            if (esriMapsWebRoot) {
                esriMapsSettings.esriMapsWebRoot = ersiMapsWebRoot;
                EsriMapsWebRoot = ersiMapsWebRoot;
                _loadScript(_getInitFilePath());
            }
            else {
                var esriMapsWebRootQS = this.getQueryStringParameter("EsriMapsWebRoot");
                if (esriMapsWebRootQS) {
                    esriMapsSettings.esriMapsWebRoot = esriMapsWebRootQS;
                    EsriMapsWebRoot = esriMapsWebRootQS;
                    _loadScript(_getInitFilePath());
                }
                else {
                    SP.SOD.executeOrDelayUntilScriptLoaded(function () {
                        _getConfigSettingsAndInitialize();
                    }, "sp.js");
                }
            }
        },

        getQueryStringParameter: function(paramToRetrieve) {
            if (document && document.URL) {
                var split = document.URL.split("?");
                if (split) {
                    var firstElement = split[1];
                    if (firstElement) {
                        var params = firstElement.split("&");
                        if (params) {
                            var strParams = "";
                            for (var i = 0; i < params.length; i = i + 1) {
                                var singleParam = params[i].split("=");
                                if (singleParam[0] == paramToRetrieve)
                                    if (paramToRetrieve === "ArcGISConnection") {
                                        return decodeURIComponent(esriMapsSettings.standardizePortalUrl(singleParam[1]));
                                    } else {
                                        return decodeURIComponent(singleParam[1]);
                                    }
                            }
                        }
                    }
                }
            }
            return null;
        },

        getQueryStringParams: function () {
            if (document && document.URL) {
                var split = document.URL.split("?");
                if (split) {
                    var firstElement = split[1];
                    if (firstElement) {
                        return firstElement.toString();
                    }
                }
            }
            return null;
        },

        setQueryParam: function (uri, key, value) {
            var regEx = new RegExp("([?&])" + key + "=.*?(&|$)", "i");
            var separator = uri.indexOf('?') !== -1 ? "&" : "?";
            if (uri.match(regEx)) {
                if (value) {
                    return uri.replace(regEx, '$1' + key + "=" + value + '$2');
                }
                else {
                    return uri.replace(regEx, "");
                }
            }
            else {
                if (value) {
                    return uri + separator + key + "=" + value;
                } else {
                    return uri;
                }
            }
        },

        getsharePointJSPath: function () {
            return esriMapsSettings.getScriptsPath() + "/sharePoint/esriMapsSharePoint";
        },

        getEmfJSPath: function () {
            return esriMapsSettings.getScriptsPath() + "/emf/";
        },

        getCommonJSPath: function () {
            return esriMapsSettings.getScriptsPath() + "/common";
        },        

        getWebRoot: function(){
            if (esriMapsSettings.esriMapsWebRoot.indexOf("http") > -1 ) {
                return esriMapsSettings.esriMapsWebRoot; // SP Online  ??NEED TO TEST!!!!!!
            } else {
                var port = document.location.port;
                var siteUrl = esriMapsSettings.getQueryStringParameter("SiteUrlRelative") || "";
                var origin = document.location.protocol + "//" + document.location.host;
                return origin + siteUrl + esriMapsSettings.esriMapsWebRoot;
            }            
        },

        getScriptsPath: function() {
            if (esriMapsSettings.esriMapsWebRoot.indexOf("http") > -1) {
                if (esriMapsSettings.isSPO()) {
                    return esriMapsSettings.esriMapsWebRoot; // SP Online     
                } else {
                    return esriMapsSettings.esriMapsWebRoot + "/scripts";  //viewExtension.aspx, ManageExtensions.aspx, WorkflowSettings.aspx, etc
                }
            } else {
                var port = document.location.port;
                var siteUrl = esriMapsSettings.getQueryStringParameter("SiteUrlRelative") || "";
                var origin = document.location.protocol + "//" + document.location.host;
                return origin + siteUrl + esriMapsSettings.esriMapsWebRoot + "/scripts";
            }
        },

        getClientContext: function () {
            var siteUrl = esriMapsSettings.getQueryStringParameter("SiteUrlRelative");
            var appweburl = esriMapsSettings.getQueryStringParameter("SPAppWebUrl");
            var hostweburl = esriMapsSettings.getQueryStringParameter("SPHostUrl");
            var isSPO = esriMapsSettings.isSPO();
            var clientContext, appContextSite, web, configList;
            if (appweburl && hostweburl) {         //SPO
                clientContext = new SP.ClientContext(appweburl);
                appContextSite = new SP.AppContextSite(clientContext, hostweburl);
                web = appContextSite.get_web();
                isSPO = true;                
            } else {                                         //SP
                if (siteUrl) {
                    clientContext = new SP.ClientContext(siteUrl);
                }
                else {
                    clientContext = new SP.ClientContext.get_current();
                }
                web = clientContext.get_web();
            }

            if (isSPO) {
                configList = clientContext.get_web().get_lists().getByTitle("EsriMapsConfigurationSettings");
            } else {
                configList = clientContext.get_site().get_rootWeb().get_lists().getByTitle("Esri Maps Configuration Settings");
            }

            return {
                clientContext: clientContext,
                web: web,
                configList: configList,
                isSPO: isSPO
            }
        },

        getIsMapEditMode: function () {
            return esriMapsSettings.getQueryStringParameter("isEditMode") === "1";       

        },

        isSPO: function () {
            if (esriMapsSettings && esriMapsSettings.getQueryStringParameter("spServerVersion") === "SharePointOnline") {
                return true;
            }
            else if (typeof _spPageContextInfo !== "undefined" && _spPageContextInfo && _spPageContextInfo.siteClientTag && _spPageContextInfo.siteClientTag.indexOf("$$16") >= 0)
                return true;
            else {
                //SPO change caused _spPageContextInfo to never be defined in Client Web Parts.
                //Use this as fallback to decide whether SPO or not. Only Map Web Part sets SPAppWebUrl. 
                var appweburl = esriMapsSettings.getQueryStringParameter("SPAppWebUrl");
                var hostweburl = esriMapsSettings.getQueryStringParameter("SPHostUrl");
                if (appweburl && hostweburl) {//SPO
                    return true;
                }
            }
            return false;
        },

        getParent: function () {
            return parent.parent;
        },

        getDocument: function () {
            return esriMapsSettings.getParent().document;
        }
    };
}());
